# BizDoc for Flutter

## Project structure

* /lib/core - Widget state, configuration
* /lib/authentication - Optional authentication methods
* /lib/models - Models
* /lib/controls - BizDoc shared controls
* /lib/services - Backend APIs
* /lib/pages - UI
* i18n - localization

## Architecture

| Name | Usage
| -- | ---
| BizDocApp | Root widget
| SplashPage | Prepare app
| ServerUriPage | Setup server uri
| SignInPage | Authentication (Credentials mode)
| HomePage | Main
| DashboardPage | Dashboard in `HomePage`
| BrowsePage | Lists messages in `HomePage`,
| ComposePage | Detail model actions
| CubeViewPage | Cube chart in `HomePage`
| ExplorePage | `CubeViewPage` explore
| NotificationsPage | Lists notifications
| Fields | Layout input from `FieldInfo` list.

The [main.dart](lib/main.dart) calls `ServerUriPage` to register server for the first time. Once set, it continues to `SignInPage` to authenticate, caching a user _Profile_ of authorized components and settings.
On a subsequent initialization, main tries to refresh the profile, without requiring authentication.

If a profile exists the main page calls `HomePage`, which shows either a `DashboardPage`, `CubeViewPage` or `BrowsePage`, based on menu selection in `HomeDrawer`.
Secondary drawer shows `ContactsWidget` that changes to `ConversationWidget` when a contact is selected. Home also navigates to `NotificationsPage`.

Creating a new message, or choosing a message from `BrowsePage`, opens `ComposePage`.

Compose page adds the _Form_ widget, which access the BizDocFormState provider to notify upon model changes.
Compose presents `CommentsWidget` on the secondary drawer, listening to the hub comments stream. It shows `AttachmentsWidget` with download.

### States

`BizDocAppState`
`BizDocFormState`
`BizDocWidgetState`

### Singletons

[Session](#lib/services/session.dart) wraps the server calls and maintaining app state.

[Hub](#lib/core/hub.dart) singleton deserialize firebase contracts and broadcast them to _events_ stream.
Home listens to hub events notifications, Browse listen to hub events messages, and Compose listen to hub events targeting the presented message.

[Users](#lib/services/accounts.dart).
[Geo](#lib/services/geo.dart).
[Cube](#lib/services/cube.dart).
[Mailbox](#lib/services/mailbox.dart).
[Chat](#lib/services/chat.dart).

[Notifications](#lib/core/notifications.service.dart)

### Implementation

This package serves as a basis for implementation, which adds custom components such as forms and dashboard tiles.

Custom components require registration in BizDOcApp main.dart.
