import 'package:bizdoc/core/app.dart';
import 'package:bizdoc/gen_l10n/app_localizations.dart';
import 'package:bizdoc/models/location.dart';
import 'package:bizdoc/services/geo.dart';
import 'package:flutter/material.dart';
import 'package:flutter_typeahead/flutter_typeahead.dart';
import 'package:provider/provider.dart';

typedef TapArgs = void Function(String address, String? placeId);

class AddressField extends StatefulWidget {
  final String? labelText;
  final String? initialValue;
  final TapArgs onChanged;
  const AddressField({
    Key? key,
    required this.onChanged,
    this.labelText,
    this.initialValue,
  }) : super(key: key);

  @override
  _AddressFieldState createState() => _AddressFieldState();
}

class _AddressFieldState extends State<AddressField> {
  @override
  Widget build(BuildContext context) {
    return Consumer<AppState>(builder: (context, appState, child) {
      return TypeAheadField<Address>(
        noItemsFoundBuilder: (context) => Container(),
        textFieldConfiguration: TextFieldConfiguration(
            autofocus: true,
            controller: TextEditingController(text: widget.initialValue),
            // style: DefaultTextStyle.of(context)
            //     .style
            //     .copyWith(fontStyle: FontStyle.italic),
            decoration: InputDecoration(
                labelText: AppLocalizations.of(context)!.address,
                suffix: const Icon(Icons.place))),
        suggestionsCallback: (value) async {
          try {
            return await geo.addresses(value);
          } catch (e) {
            print(e);
          }
          return List.empty();
        },
        itemBuilder: (context, item) => ListTile(
          title: Text(item.address),
        ),
        onSuggestionSelected: (item) async {
          setState(() {
            widget.onChanged(item.address, item.placeId);
          });
        },
      );
    });
  }
}
