import 'package:flutter/material.dart';
import 'package:flutter_typeahead/flutter_typeahead.dart';

import '../services.dart';

typedef TapArgs = void Function(String key, String? value);

class AutocompleteField extends StatefulWidget {
  final String dataType;
  final String? initialValue;
  final String? labelText;
  final TapArgs onChanged;
  const AutocompleteField(
      {Key? key,
      required this.onChanged,
      required this.dataType,
      this.initialValue,
      this.labelText})
      : super(key: key);

  @override
  _AutocompleteFieldState createState() => _AutocompleteFieldState();
}

class _AutocompleteFieldState extends State<AutocompleteField> {
  @override
  Widget build(BuildContext context) {
    return TypeAheadField<MapEntry>(
      textFieldConfiguration: TextFieldConfiguration(
          controller: TextEditingController(text: widget.initialValue),
          decoration: InputDecoration(
            labelText: widget.labelText,
          )),
      suggestionsCallback: (value) async =>
          (await dataSource.map(widget.dataType)).entries,
      itemBuilder: (context, item) => ListTile(
        title: Text(item.key),
        subtitle: Text(item.value),
      ),
      onSuggestionSelected: (item) {
        widget.onChanged(item.key, item.value);
      },
    );
  }
}
