abstract class NotificationBase {
  final int id;
  final int? recipientId;
  DateTime time;
  NotificationBase(this.id, this.time, this.recipientId);
  static construct(Map<String, dynamic> json) {
    switch (json['type']) {
      case 'CubeAnomaly':
        return CubeAnomaly.fromJson(json);
      case 'StateChanged':
        return StateChanged.fromJson(json);
      case 'Escalated':
        return Escalated.fromJson(json);
      case 'Nudge':
        return Nudge.fromJson(json);
      case 'UpcomingEvent':
        return UpcomingEvent.fromJson(json);
      case 'Tagged':
        return Tagged.fromJson(json);
      case 'Liked':
        return Liked.fromJson(json);
      case 'Commented':
        return Commented.fromJson(json);
      case 'TextMessage':
        return TextMessage.fromJson(json);
      default:
        throw 'Unexpected notification ${json['type']}';
    }
  }

  void update(NotificationBase model) {
    time = model.time;
  }
}

class CubeAnomaly extends NotificationBase {
  final String cube;
  final List<String> axes;
  final double value;
  final List<String> numbers;
  CubeAnomaly(int id, DateTime time, int? recipientId, this.cube, this.axes,
      this.value, this.numbers)
      : super(id, time, recipientId);
  factory CubeAnomaly.fromJson(Map<String, dynamic> json) {
    return CubeAnomaly(
        json['id'],
        DateTime.parse(json['sent']),
        json['recipientId'],
        json['cube'],
        List.from(json['axes']),
        json['value'],
        json['numbers']);
  }
}

class StateChanged extends NotificationBase {
  final String number;
  final String state;
  StateChanged(int id, DateTime time, int? recipientId, this.number, this.state)
      : super(id, time, recipientId);
  factory StateChanged.fromJson(Map<String, dynamic> json) {
    return StateChanged(json['id'], DateTime.parse(json['sent']),
        json['recipientId'], json['number'], json['state']);
  }
}

class Nudge extends NotificationBase {
  final String number;
  final DateTime expire;
  Nudge(int id, DateTime time, int? recipientId, this.number, this.expire)
      : super(id, time, recipientId);
  factory Nudge.fromJson(Map<String, dynamic> json) {
    return Nudge(json['id'], DateTime.parse(json['sent']), json['recipientId'],
        json['number'], DateTime.parse(json['expire']));
  }
}

class Liked extends NotificationBase {
  final String number;
  final List<String> userIds;
  final int? voted;
  Liked(int id, DateTime time, int? recipientId, this.number, this.userIds,
      this.voted)
      : super(id, time, recipientId);
  factory Liked.fromJson(Map<String, dynamic> json) {
    return Liked(json['id'], DateTime.parse(json['sent']), json['recipientId'],
        json['number'], List.from(json['userIds']), json['voted']);
  }
}

class Escalated extends NotificationBase {
  final String number;
  final String? role;
  final String substitutingId;
  final Duration duration;
  Escalated(int id, DateTime time, int? recipientId, this.number, this.role,
      this.substitutingId, this.duration)
      : super(id, time, recipientId);
  factory Escalated.fromJson(Map<String, dynamic> json) {
    return Escalated(
        json['id'],
        DateTime.parse(json['sent']),
        json['recipientId'],
        json['number'],
        json['role'],
        json['substitutingId'],
        Duration(seconds: json['duration']));
  }
}

class Commented extends NotificationBase {
  final String number;
  final String? text;
  final List<String> userIds;
  Commented(int id, DateTime time, int? recipientId, this.number, this.userIds,
      this.text)
      : super(id, time, recipientId);
  factory Commented.fromJson(Map<String, dynamic> json) {
    return Commented(
        json['id'],
        DateTime.parse(json['sent']),
        json['recipientId'],
        json['number'],
        List.from(json['userIds']),
        json['text']);
  }
}

class TextMessage extends NotificationBase {
  final String text;
  TextMessage(int id, DateTime time, int? recipientId, this.text)
      : super(id, time, recipientId);
  factory TextMessage.fromJson(Map<String, dynamic> json) {
    return TextMessage(json['id'], DateTime.parse(json['sent']),
        json['recipientId'], json['text']);
  }
}

class UpcomingEvent extends NotificationBase {
  final String number;
  final DateTime starting;
  UpcomingEvent(
      int id, DateTime time, int? recipientId, this.number, this.starting)
      : super(id, time, recipientId);
  factory UpcomingEvent.fromJson(Map<String, dynamic> json) {
    return UpcomingEvent(json['id'], DateTime.parse(json['sent']),
        json['recipientId'], json['number'], DateTime.parse(json['starting']));
  }
}

class Tagged extends NotificationBase {
  final String number;
  final String who;
  Tagged(int id, DateTime time, int? recipientId, this.number, this.who)
      : super(id, time, recipientId);
  factory Tagged.fromJson(Map<String, dynamic> json) {
    return Tagged(json['id'], DateTime.parse(json['sent']), json['recipientId'],
        json['number'], json['who']);
  }
}
