import 'notifications.dart';

class Profile {
  final List<FormInfo> forms;
  final List<CubeInfo> cubes;
  final List<ActionInfo> actions;
  final List<FolderInfo> folders;
  final List<WidgetInfo> widgets;
  final List<NotificationBase> notifications;
  final List<ContactInfo> contacts;
  final List<StateInfo> states;
  int notificationsCount;
  int conversationsCount;
  int inboxCount;
  final String userId;
  final String? byId;
  final String name;
  final String? byName;
  final LicenseInfo license;
  Profile(
    this.forms,
    this.actions,
    this.cubes,
    this.folders,
    this.widgets,
    this.notifications,
    this.notificationsCount,
    this.conversationsCount,
    this.inboxCount,
    this.contacts,
    this.userId,
    this.byId,
    this.name,
    this.byName,
    this.states,
    this.license,
  );
  factory Profile.fromJson(Map<String, dynamic> json) {
    return Profile(
        List<FormInfo>.from(json['forms'].map((e) => FormInfo.fromJson(e))),
        List<ActionInfo>.from(
            json['actions'].map((e) => ActionInfo.fromJson(e))),
        List<CubeInfo>.from(json['cubes'].map((e) => CubeInfo.fromJson(e))),
        List<FolderInfo>.from(
            json['folders'].map((e) => FolderInfo.fromJson(e))),
        List<WidgetInfo>.from(
            json['widgets'].map((e) => WidgetInfo.fromJson(e))),
        List<NotificationBase>.from(
            json['notifications'].map((e) => NotificationBase.construct(e))),
        json['notificationsCount'],
        json['conversationsCount'],
        json['inboxCount'],
        List<ContactInfo>.from(
            json['contacts'].map((e) => ContactInfo.fromJson(e))),
        json['userId'],
        json['byId'],
        json['name'],
        json['byName'],
        List<StateInfo>.from(json['states'].map((e) => StateInfo.fromJson(e))),
        LicenseInfo.fromJson(json['license']));
  }
}

class LicenseInfo {
  final LicenseMode mode;
  final DateTime? expiration;
  final String companyName;

  LicenseInfo(this.mode, this.expiration, this.companyName);
  factory LicenseInfo.fromJson(Map<String, dynamic> json) {
    return LicenseInfo(parseMode(json['mode']),
        DateTime.parse(json['expiration']), json['companyName']);
  }
  static LicenseMode parseMode(String mode) {
    switch (mode) {
      case 'Expired':
        return LicenseMode.expired;
      case 'Limited':
        return LicenseMode.limited;
      case 'Grace':
        return LicenseMode.grace;
      default:
        return LicenseMode.valid;
    }
  }
}

enum LicenseMode { valid, grace, limited, expired }

class FormInfo {
  final String name;
  final String title;
  final String? icon;
  final bool creatable;
  final PositionPolicy position;
  final String template;
  FormInfo(this.name, this.title, this.icon, this.creatable, this.position,
      this.template);
  factory FormInfo.fromJson(Map<String, dynamic> json) {
    return FormInfo(json['name'], json['title'], json['icon'],
        json['creatable'], mapPosition(json['position']), json['template']);
  }

  static PositionPolicy mapPosition(String? json) {
    switch (json) {
      case 'Ask':
        return PositionPolicy.ask;
      case 'Require':
        return PositionPolicy.require;
      default:
        return PositionPolicy.none;
    }
  }
}

class FolderInfo {
  static const inbox = "ib";
  final String name;
  final String title;
  final String? icon;
  FolderInfo(this.name, this.title, this.icon);
  factory FolderInfo.fromJson(Map<String, dynamic> json) =>
      FolderInfo(json['name'], json['title'], json['icon']);
}

class WidgetInfo {
  final String name;
  final String? title;
  final String? guide;
  final String template;
  final Map<String, dynamic>? options;
  WidgetInfo(this.name, this.title, this.guide, this.template, this.options);
  factory WidgetInfo.fromJson(Map<String, dynamic> json) => WidgetInfo(
      json['name'],
      json['title'],
      json['guide'],
      json['template'],
      json['options']);
}

class ActionInfo {
  final String name;
  final String title;
  final String? icon;
  final int? group;
  final String? adjective;
  final String? verb;
  final String? past;
  final String? you;
  final List<FieldInfo> arguments;

  final bool? multiple;
  ActionInfo(this.name, this.title, this.icon, this.group, this.arguments,
      this.adjective, this.verb, this.past, this.you, this.multiple);
  factory ActionInfo.fromJson(Map<String, dynamic> json) {
    return ActionInfo(
      json['name'],
      json['title'],
      json['icon'],
      json['group'],
      List<FieldInfo>.from(json['arguments'].map((e) => FieldInfo.fromJson(e))),
      json['adjective'],
      json['verb'],
      json['past'],
      json['you'],
      json['multiple'],
    );
  }
}

class CubeInfo {
  final String name;
  final String title;
  final String? currencyCode;
  final List<CubeAxis> axes;
  final List<CubeView> views;
  final List<CubeIndex> indices;
  final YAxis? y;
  CubeInfo(this.name, this.title, this.axes, this.views, this.currencyCode,
      this.y, this.indices);
  factory CubeInfo.fromJson(Map<String, dynamic> json) {
    return CubeInfo(
      json['name'],
      json['title'],
      List<CubeAxis>.from(json['axes'].map((e) => CubeAxis.fromJson(e))),
      List<CubeView>.from(json['views'].map((e) => CubeView.fromJson(e))),
      json['currencyCode'],
      json.containsKey('yAxis') ? YAxis(json['yAxis']['title']) : null,
      List<CubeIndex>.from(json['indices'].map((e) => CubeIndex.fromJson(e))),
    );
  }
}

class YAxis {
  final String? title;
  YAxis(this.title);
}

class CubeAxis {
  final String name;
  final String? title;
  final String dataType;
  final bool combination;
  final SelectionMode? selectionMode;
  CubeAxis(this.name, this.title, this.dataType, this.combination,
      this.selectionMode);
  factory CubeAxis.fromJson(Map<String, dynamic> json) {
    return CubeAxis(
        json['name'],
        json['title'],
        json['dataType'],
        json.containsKey('combination') && json['combination'],
        mapSelectionMode(json['selectionMode']));
  }

  static mapSelectionMode(String? json) {
    switch (json) {
      case 'Multiple':
        return SelectionMode.multiple;
      case 'Pattern':
        return SelectionMode.pattern;
      case 'Node':
        return SelectionMode.none;
      default:
        return SelectionMode.single;
    }
  }
}

class CubeIndex {
  final String name;
  final String title;
  CubeIndex(this.name, this.title);
  factory CubeIndex.fromJson(Map<String, dynamic> json) {
    return CubeIndex(json['name'], json['title']);
  }
}

enum SelectionMode { single, multiple, pattern, none }
enum ViewType { chart, grid, pivot, spreadsheet }

class CubeView {
  final String name;
  final String title;
  final List<String> xAxis;
  final List<String>? series;
  final List<String>? indices;
  final ChartType? chartType;
  final ViewType type;

  CubeView(this.name, this.title, this.xAxis, this.series, this.type,
      this.chartType, this.indices);
  factory CubeView.fromJson(Map<String, dynamic> json) {
    return CubeView(
        json['name'],
        json['title'],
        json['xAxis'] is String
            ? <String>[json['xAxis'] as String]
            : List.from(json['xAxis']),
        json.containsKey('series')
            ? json['series'] is String
                ? <String>[json['series'] as String]
                : List.from(json['series'])
            : [],
        _parseViewType(json['type']),
        mapChartType(json['chartType']),
        List.from(json['indices'] ?? []));
  }
  static _parseViewType(String? json) {
    switch (json) {
      case 'Grid':
        return ViewType.grid;
      case 'Pivot':
        return ViewType.pivot;
      case 'Spreadsheet':
        return ViewType.spreadsheet;
      default:
        return ViewType.chart;
    }
  }

  static ChartType? mapChartType(String? json) {
    switch (json) {
      case 'Bar':
        return ChartType.bar;
      case 'StackingBar':
        return ChartType.stackingBar;
      case 'StackingBar100':
        return ChartType.stackingBar100;
      case 'Column':
        return ChartType.column;
      case 'StackingColumn':
        return ChartType.stackingColumn;
      case 'StackingColumn100':
        return ChartType.stackingColumn100;
      case 'Doughnut':
        return ChartType.doughnut;
      case 'HalfDoughnut':
        return ChartType.halfDoughnut;
      case 'Pie':
        return ChartType.pie;
      case 'Area':
        return ChartType.area;
      case 'StackingArea':
        return ChartType.stackingArea;
      case 'StackingArea100':
        return ChartType.stackingArea100;
      case 'Line':
        return ChartType.line;
      case 'Spline':
        return ChartType.spline;
      case 'SplineArea':
        return ChartType.splineArea;
      case 'Polar':
        return ChartType.polar;
      case 'Radar':
        return ChartType.radar;
      case 'StepLine':
        return ChartType.stepLine;
      case 'StepArea':
      case 'StackingStepArea':
        return ChartType.stepArea;
      case 'Pyramid':
        return ChartType.pyramid;
      case 'Funnel':
        return ChartType.funnel;
      case 'Scatter':
        return ChartType.scatter;
      case 'Bubble':
        return ChartType.bubble;
      default:
        return null;
    }
  }
}

enum PositionPolicy { none, ask, require }

enum ChartType {
  pie,
  bar,
  bubble,
  stackingBar,
  stackingBar100,
  line,
  spline,
  area,
  splineArea,
  doughnut,
  halfDoughnut,
  column,
  stackingArea,
  stackingArea100,
  stackingColumn,
  stackingColumn100,
  scatter,
  radar,
  stepLine,
  stepArea,
  stackingLine,
  stackingLine100,
  funnel,
  polar,
  pyramid
}

class StateInfo {
  final String name;
  final String title;
  StateInfo(this.name, this.title);
  factory StateInfo.fromJson(Map<String, dynamic> json) {
    return StateInfo(json['name'], json['title']);
  }
}

class RoleInfo {
  final String name;
  final String title;
  RoleInfo(this.name, this.title);
  factory RoleInfo.fromJson(Map<String, dynamic> json) {
    return RoleInfo(json['name'], json['title']);
  }
}

class FieldInfo {
  final String name;
  final String? label;
  final FieldKind kind;
  final String source;

  FieldInfo(this.name, this.label, this.kind, this.source);
  factory FieldInfo.fromJson(Map<String, dynamic> json) {
    return FieldInfo(
      json['name'],
      json['label'],
      mapKind(json['kind']),
      json['source'],
    );
  }
  static FieldKind mapKind(String? kind) {
    switch (kind) {
      case 'Number':
        return FieldKind.number;
      case 'Currency':
        return FieldKind.currency;
      case 'Select':
        return FieldKind.select;
      case 'Autocomplete':
        return FieldKind.autocomplete;
      case 'Date':
        return FieldKind.date;
      case 'Switch':
        return FieldKind.switchKind;
      case 'Check':
        return FieldKind.check;
      case 'EmailAddress':
        return FieldKind.emailAddress;
      case 'Url':
        return FieldKind.url;
      case 'PhoneNumber':
        return FieldKind.phoneNumber;
      default:
        return FieldKind.string;
    }
  }
}

enum FieldKind {
  string,
  number,
  date,
  dateTime,
  check,
  currency,
  percent,
  // Time,
  // Html,
  // File,
  // Duration,
  // DateRange,
  autocomplete,
  url,
  address,
  select,
  emailAddress,
  text,
  password,
  phoneNumber,
  switchKind,
}

class ContactInfo {
  final String id;
  late String name;
  Map<String, dynamic>? resource;
  String? text;
  DateTime time;
  DateTime? watermark;
  DateTime? visited;
  bool accepted;
  int count;
  bool? forgotten;
  ContactInfo(this.id, this.time, this.accepted, this.count,
      {this.resource, this.text, this.watermark, this.visited, this.forgotten});
  factory ContactInfo.fromJson(Map<String, dynamic> json) {
    return ContactInfo(
      json['id'],
      DateTime.parse(json['time']),
      json['accepted'],
      json['count'],
      resource: json['resource'],
      text: json['text'],
      watermark: json.containsKey('watermark')
          ? DateTime.parse(json['watermark'])
          : null,
      visited:
          json.containsKey('visited') ? DateTime.parse(json['visited']) : null,
      forgotten: json['forgotten'],
    );
  }

  factory ContactInfo.from(UserInfo item) {
    return ContactInfo(item.id, DateTime.now().toUtc(), false, 0);
  }
}

class PatternInfo {
  final String title;
  final Map<String, dynamic> axes;
  PatternInfo(this.title, this.axes);
  factory PatternInfo.fromJson(Map<String, dynamic> json) {
    return PatternInfo(json['title'], json['axes']);
  }
}

class CurrencyInfo {
  final String name;
  final String code;
  final String symbol;
  Map<String, dynamic> asJson() =>
      {'value': code, 'label': symbol + ' ' + name};
  CurrencyInfo(this.name, this.code, this.symbol);
  factory CurrencyInfo.fromJson(Map<String, dynamic> json) {
    return CurrencyInfo(json['name'], json['code'], json['symbol']);
  }
}

class UserInfo {
  final String id;
  final String name;
  final String? photo;
  UserInfo(this.id, this.name, this.photo);
  factory UserInfo.fromJson(Map<String, dynamic> json) {
    return UserInfo(json['id'], json['name'], json['photo']);
  }
}

enum Gender { male, female }

enum AvailabilityMode { offline, online, outOfOffice, vacation }
