import 'package:flutter/material.dart';
import 'package:open_file/open_file.dart';
import 'package:stacked/stacked.dart';

import 'package:bizdoc/services/mailbox.dart';
import 'package:bizdoc/models/data.dart';

/// presents attachments on [ComposePage], download and open
class AttachmentsWidget extends StatefulWidget {
  final RecipientModel model;
  const AttachmentsWidget(this.model, {Key? key}) : super(key: key);
  @override
  _AttachmentsWidgetState createState() => _AttachmentsWidgetState();
}

class _AttachmentsWidgetState extends State<AttachmentsWidget> {
  @override
  Widget build(BuildContext context) {
    return SingleChildScrollView(
      scrollDirection: Axis.vertical,
      child: Row(
        children: [
          for (var item in widget.model.attachments
              .where((element) => element.deleted == null || !element.deleted!))
            ViewModelBuilder<ProgressModel>.reactive(
                builder: (_, progress, widget) {
                  return InputChip(
                    key: Key(item.id.toString()),
                    selected: false,
                    onPressed: () => _download(item, progress),
                    label: Row(
                      children: [
                        if (progress.loading)
                          CircularProgressIndicator(value: progress.value),
                        Text(item.fileName),
                      ],
                    ),
                    onDeleted: () => _delete(item),
                  );
                },
                viewModelBuilder: () => ProgressModel()),
        ],
      ),
    );
  }

  _delete(Attachment item) async {
    try {
      await mailbox.deleteAttachment(
          widget.model.id, item.id, widget.model.version);
      widget.model.attachments.remove(item);
      setState(() {});
    } catch (e) {}
  }

  _download(Attachment item, ProgressModel progress) async {
    try {
      final file = await mailbox.download(
        widget.model.documentId,
        item.id,
        progress: (p1, p2) => progress.assign(p2 / p1 * 100),
      );
      OpenFile.open(file);
    } catch (e) {
      print(e);
    } finally {
      progress.finish();
    }
  }
}

class ProgressModel extends ChangeNotifier {
  double? _value;
  double? get value => _value;
  bool get loading => _value != null;
  void assign(double? value) {
    _value = value;
    notifyListeners();
  }

  void finish() {
    assign(null);
  }
}
