import 'package:flutter/material.dart';
import 'package:flutter/scheduler.dart';

import 'package:bizdoc/gen_l10n/app_localizations.dart';
import 'package:bizdoc/services/session.dart';

/// prompt server url, ping server and return 'true' to navigation origin.
class ServerUrlPage extends StatefulWidget {
  static const routeName = 'serverUrl';

  const ServerUrlPage({Key? key}) : super(key: key);

  @override
  _ServerUrlPageState createState() => _ServerUrlPageState();
}

class _ServerUrlPageState extends State<ServerUrlPage> {
  final _controller = TextEditingController();
  bool _pinging = false;
  bool _failed = false;
  @override
  Widget build(BuildContext context) {
    if (_pinging) {
      return const Center(child: CircularProgressIndicator());
    }
    return Scaffold(
      body: Column(
        crossAxisAlignment: CrossAxisAlignment.stretch,
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          TextFormField(
            autofocus: true,
            keyboardType: TextInputType.url,
            onSaved: (_) => _ping(),
            decoration: InputDecoration(
                labelText: AppLocalizations.of(context)!.serverAddress,
                errorText: _failed
                    ? AppLocalizations.of(context)!.serverAddressErr
                    : null),
            controller: _controller,
          ),
          ElevatedButton(
            onPressed: _ping,
            child: Text(AppLocalizations.of(context)!.oK),
          )
        ],
      ),
    );
  }

  _togglePinging(bool val) => setState(() => _pinging = val);

  void _ping() async {
    _togglePinging(true);
    try {
      /// ping server
      await session.setup(_controller.text);
      SchedulerBinding.instance!.addPostFrameCallback((_) {
        Navigator.pop(context, true);
      });

      /// return 'true'
    } catch (e) {
      _failed = true;
      _togglePinging(false);
    }
  }
}
