import 'dart:io';
import 'dart:ui';

import 'package:dio/adapter.dart';
import 'package:dio/dio.dart';
import 'package:flutter_secure_storage/flutter_secure_storage.dart';
import 'package:permission_handler/permission_handler.dart';

import '../core/app.dart';

class Network {
  final _secureStorage = const FlutterSecureStorage();
  late AppState appState;
  final _dio = Dio();
  get dio => _dio;
  Network();
  initialize(AppState appState) async {
    this.appState = appState;
    final systemLocales = window.locales.map((element) =>
        element.countryCode != null
            ? '${element.languageCode}-${element.countryCode}'
            : element.languageCode);
    _dio.options.baseUrl = appState.serverAddress! + '/api';
    final Map<String, String> headers = {
      HttpHeaders.acceptLanguageHeader: systemLocales.join(', '),
      HttpHeaders.acceptHeader: 'application/json, text/plain, */*',
      'Access-Control-Allow-Origin': '*',
    };
    await Permission.storage.request();
    // Permission.requestPermissions([PermissionName.Storage]);
    //Directory appDocDir = await getApplicationDocumentsDirectory();
    final token = await appState.authentication.getAccessToken();
    _dio.options.headers['Authorization'] = 'Bearer $token';
    final connectionId = await _secureStorage.read(key: 'connectionId');
    if (connectionId != null) headers['connection-id'] = connectionId;
    _dio.options.headers = headers;
    _dio.options.contentType = Headers.jsonContentType;
    (_dio.httpClientAdapter as DefaultHttpClientAdapter).onHttpClientCreate =
        (HttpClient client) {
      client.badCertificateCallback =
          (X509Certificate cert, String host, int port) => true;
      return client;
    };
  }
}

final network = Network();
